@@ -19,6 +19,7 @@
 
 #include <src/outils/outils.h>
 #include <src/machine/extensions/analyseur/analyseur.h>
+#include <src/global/installation.h>
 
 using namespace SVM::Outils;
 
@@ -258,10 +259,18 @@ std::string Generation::makefile(const Langage& langage, const SVM::Outils::Exte
 		oss	<< liste_tests.str();
 	}
 	oss	<< std::endl
-		<< "\t@echo; echo \"Test suite:\"; for t in" << liste_tests.str() << "; do echo -n \"  \"; tail -n 1 $$t; done; if [ " << extension->_tests.size() << " -eq 0 ] ; then echo \"No tests.\"; else ok=$$(tail -n 1" << liste_tests.str() << "| grep -c 'OK$$'); sk=$$(tail -n 1" << liste_tests.str() << "| grep -c 'SKIPPED$$'); echo \"Result: $$ok/$$(( " << extension->_tests.size() << " - $$sk)) passed.\" ; fi; echo;" << std::endl
-		<< "" << std::endl
+		<< "\t@echo; echo \"Test suite:\"; ";
+	if(extension->_tests.empty())
+	{
+		oss	<< "echo \"No tests.\"" << std::endl;
+	}
+	else
+	{
+		oss	<< "for t in" << liste_tests.str() << "; do echo -n \"  \"; tail -n 1 $$t; done; ok=$$(tail -n 1" << liste_tests.str() << "| grep -c 'OK$$'); sk=$$(tail -n 1" << liste_tests.str() << "| grep -c 'SKIPPED$$'); echo \"Result: $$ok/$$(( " << extension->_tests.size() << " - $$sk)) passed.\" ; echo; [ $$ok -eq $$(( " << extension->_tests.size() << " - $$sk)) ]" << std::endl;
+	}
+	oss	<< "" << std::endl
 		<< "clean:" << std::endl
-		<< "	rm -rf src/plugin.o $(DEPENDENCIES) $(GENERATED) libsvm" << extension->_nom << ".so" << liste_tests.str() << std::endl
+		<< "	@echo \"Cleaning generated files...\"; rm -rf src/plugin.o $(DEPENDENCIES) $(GENERATED) libsvm" << extension->_nom << ".so" << liste_tests.str() << std::endl
 		<< "" << std::endl
 		<< "dist: all" << std::endl
 		<< "	@echo \"Cleaning build system...\" ; rm -rf configure.ac Makefile.* */Makefile.* plugin_install update_patches src test plugin_configuration_string 2>/dev/null" << std::endl
@@ -751,22 +760,7 @@ void Fichiers::commente(const std::string& texte, const Commentaire& commentaire
 
 void Fichier::echappe_shell()
 {
-	std::string r;
-	for(auto &c: _valeur)
-	{
-		if(c=='$')
-		{
-			r += "\\$";
-			continue;
-		}
-		if(c=='\\')
-		{
-			r += "\\\\";
-			continue;
-		}
-		r += c;
-	}
-	_valeur = r;
+	_valeur = SVM::Global::Chaine::echappe_shell(_valeur);
 }
 
 void Fichier::lignes_source(const std::string& nom)
@@ -1060,3 +1054,444 @@ std::string Generation::update_patches()
 		<< std::endl;
 	return oss.str();
 }
+
+// svm_package
+
+ConfigurationPaquet::ConfigurationPaquet()
+{
+	const auto& configuration = ConfigurationPaquet::configuration();
+	for(const auto& c: configuration)
+	{
+		_etat.insert(std::make_pair(c.first,c.second._defaut));
+	}
+}
+
+void ConfigurationPaquet::modifie(const AttributPaquet& a, const size_t position)
+{
+	const auto& configuration = ConfigurationPaquet::configuration();
+	auto cfg = configuration.find(a._nom)->second;
+	std::vector<ValeurOptionSP> valeurs;
+	for(size_t i=0 ; i<a._valeurs.size() ; ++i)
+	{
+		auto valeur = a._valeurs[i];
+		while(valeur->type()==ValeurOption::Type::VARIABLE)
+		{
+			ValeurOptionVariableSP var = std::static_pointer_cast<ValeurOptionVariable>(valeur);
+			auto itv = _variables.find(var->_nom);
+			if(itv==_variables.end())
+			{
+				std::ostringstream oss;
+				oss << "Variable " << var->_nom << " at position " << (i+1) << " does not exist.";
+				Erreur e(oss.str());
+				e.position(position,position);
+				throw e;
+			}
+			auto v = itv->second;
+			if(v->type()==ValeurOption::Type::DEFAUT)
+			{
+				v = cfg._defaut[i];
+			}
+			if(not ValeurOption::compatible(cfg._types[i],v->type()))
+			{
+				std::ostringstream oss;
+				oss << "Variable " << var->_nom << " at position " << (i+1) << " has a value of type " << ValeurOption::type(v->type()) << " instead of one among [";
+				for(const auto&t : cfg._types[i])
+				{
+					oss << " " << ValeurOption::type(t);
+				}
+				oss << " ].";
+				Erreur e(oss.str());
+				e.position(position,position);
+				throw e;
+			}
+			valeur = v;
+		}
+		if(valeur->type()==ValeurOption::Type::DEFAUT)
+		{
+			valeurs.push_back(cfg._defaut[i]);
+		}
+		else
+		{
+			valeurs.push_back(valeur);
+		}
+	}
+	_etat[a._nom] = valeurs;
+}
+
+ValeurOptionSP ConfigurationPaquet::lit(const std::string& nom, const size_t indice_valeur) const
+{
+	auto ic = _etat.find(nom);
+	if(ic==_etat.end()) throw;
+	if(indice_valeur>=ic->second.size()) throw;
+	return ic->second[indice_valeur];
+}
+
+long int ConfigurationPaquet::lit_entier(const std::string& nom, const size_t indice_valeur) const
+{
+	auto r = std::dynamic_pointer_cast<ValeurOptionEntier>(lit(nom,indice_valeur));
+	if(not r) throw;
+	return r->_valeur;
+}
+
+std::string ConfigurationPaquet::lit_chaine(const std::string& nom, const size_t indice_valeur) const
+{
+	auto r = std::dynamic_pointer_cast<ValeurOptionChaine>(lit(nom,indice_valeur));
+	if(not r) throw;
+	return r->_valeur;
+}
+
+bool ConfigurationPaquet::lit_booleen(const std::string& nom, const size_t indice_valeur) const
+{
+	auto r = std::dynamic_pointer_cast<ValeurOptionBooleen>(lit(nom,indice_valeur));
+	if(not r) throw;
+	return r->_valeur;
+}
+
+void ConfigurationPaquet::variable(const AttributPaquet& a)
+{
+	_variables[a._nom] = a._valeurs[0];
+}
+
+void AttributPaquet::verification(const std::string& type, const size_t position) const
+{
+	const auto& configuration = ConfigurationPaquet::configuration();
+	auto it = configuration.find(_nom);
+	if(it==configuration.end() or (it->second._type!=type))
+	{
+		std::ostringstream oss;
+		oss << "Package " << type << " " << _nom << " does not exist.";
+		Erreur e(oss.str());
+		e.position(position,position);
+		throw e;
+	}
+	const auto& types = it->second._types;
+	if(_valeurs.size() != types.size())
+	{
+		std::ostringstream oss;
+		oss << "Package " << type << " " << _nom << " requires " << types.size() << " argument" << ((types.size()==1)?"":"s") << ".";
+		Erreur e(oss.str());
+		e.position(position,position);
+		throw e;
+	}
+	for(size_t i=0 ; i<types.size() ; ++i)
+	{
+		if(not ValeurOption::compatible(types[i],_valeurs[i]->type()))
+		{
+			std::ostringstream oss;
+			oss << "Package " << type << " " << _nom << " requires a value at position " << (i+1) << " among [";
+			for(const auto&t : types[i])
+			{
+				oss << " " << ValeurOption::type(t);
+			}
+			oss << " ] (" << ValeurOption::type(_valeurs[i]->type()) << " provided).";
+			Erreur e(oss.str());
+			e.position(position,position);
+			throw e;
+		}
+	}
+}
+
+void OptionPaquet::verification()
+{
+	AttributPaquet::verification("option",_position);
+}
+
+void ContenuPaquetApplication::verification()
+{
+	FichierPaquet::verification(_position);
+}
+
+void ContenuPaquetBibliotheque::verification()
+{
+	FichierPaquet::verification(_position);
+}
+
+void ContenuPaquetExtension::verification()
+{
+	FichierPaquet::verification(_position);
+}
+
+void ContenuPaquetFichier::verification()
+{
+	FichierPaquet::verification(_position);
+}
+
+void FichierPaquet::verification(const size_t position)
+{
+	_fichier.open(_nom.c_str(),std::ios_base::in);
+	if(not _fichier.is_open())
+	{
+		std::ostringstream oss;
+		oss << "File " << _nom << " does not exist or is not readable.";
+		Erreur e(oss.str());
+		e.position(position,position);
+		throw e;
+	}
+}
+
+void ContenuPaquetConfiguration::verification()
+{
+	AttributPaquet::verification("set",_position);
+}
+
+void OptionPaquet::construction(ConfigurationPaquet& configuration)
+{
+	configuration.modifie(*this,_position);
+}
+
+void FichierPaquet::construction(ConfigurationPaquet& configuration, const size_t position)
+{
+	_chemin = configuration.lit_chaine("path",0);
+	if((_chemin == "..") or (_chemin.substr(0,3) == "../") or ((_chemin.size()>=3) and (_chemin.substr(_chemin.size()-3,3) == "/..")) or (_chemin.find("/../",0) != std::string::npos))
+	{
+		std::ostringstream oss;
+		oss << "File path " << _chemin << " shall not contain references to folder '..'.";
+		Erreur e(oss.str());
+		e.position(position,position);
+		throw e;
+	}
+	_fin_de_fichier = configuration.lit_chaine("end_of_file",0);
+	_destination = _chemin + "/$(basename " + _nom + ")";
+}
+
+void ContenuPaquetApplication::construction(ConfigurationPaquet& configuration)
+{
+	FichierPaquet::construction(configuration,_position);
+	_changer_interpreteur = configuration.lit_booleen("app_change_sheebang",0);
+	_racine_extensions_locales = configuration.lit_chaine("app_local_plugins_root",0);
+}
+
+void ContenuPaquetBibliotheque::construction(ConfigurationPaquet& configuration)
+{
+	FichierPaquet::construction(configuration,_position);
+}
+
+void ContenuPaquetExtension::construction(ConfigurationPaquet& configuration)
+{
+	FichierPaquet::construction(configuration,_position);
+	_installation = configuration.lit_booleen("plugin_install",0);
+	_garde_construction = configuration.lit_booleen("plugin_keep_build",0);
+	_joue_tests = configuration.lit_booleen("plugin_check",0);
+}
+
+void ContenuPaquetFichier::construction(ConfigurationPaquet& configuration)
+{
+	FichierPaquet::construction(configuration,_position);
+}
+
+void ContenuPaquetConfiguration::construction(ConfigurationPaquet& configuration)
+{
+	configuration.modifie(*this,_position);
+}
+
+void ContenuPaquetVariable::construction(ConfigurationPaquet& configuration)
+{
+	configuration.variable(*this);
+}
+
+void ContenuPaquetCode::construction(ConfigurationPaquet& configuration)
+{
+}
+
+void FichierPaquet::generation(std::ostream& os)
+{
+	os	<< "log \"Installing " << _destination << "\"" << std::endl
+		<< "mkdir -p " << _chemin << std::endl
+		<< "cat > " << _destination << " << " << _fin_de_fichier << std::endl; 
+	for(std::string ligne ; std::getline(_fichier, ligne) ; )
+	{
+		os << SVM::Global::Chaine::echappe_shell(ligne) << std::endl;
+	}
+	os	<< _fin_de_fichier << std::endl; 
+}
+
+void ContenuPaquetApplication::generation(std::ostream& os)
+{
+	FichierPaquet::generation(os);
+	os	<< "chmod u+x " << _destination << std::endl;
+	if(_changer_interpreteur)
+	{
+		os	<< "head -n 1 " << _destination << " | grep '^#!' > /dev/null 2>/dev/null && sed -i \"1c#!${SVM_PATH}/svm\" " << _destination << std::endl;
+	}
+	if(not _racine_extensions_locales.empty())
+	{
+		os	<< "sed -i 's/^[ \t]*LOCAL PLUGIN \"/&" << _racine_extensions_locales << "\\//' " << _destination << std::endl;
+	}
+}
+
+void ContenuPaquetBibliotheque::generation(std::ostream& os)
+{
+	FichierPaquet::generation(os);
+}
+
+void ContenuPaquetExtension::generation(std::ostream& os)
+{
+	FichierPaquet::generation(os);
+	if(_installation)
+	{
+		os	<< "(" << std::endl
+			<< "cd " << _chemin << std::endl
+			<< "PLUGIN=$(mktemp -p .)" << std::endl
+			<< "clean()" << std::endl
+			<< "{" << std::endl
+			<< "	rm $PLUGIN" << std::endl
+			<< "}" << std::endl
+			<< "trap clean EXIT" << std::endl
+			<< "${SVM_PATH}/svm_plugin -fi $(basename " << _nom << ") -o $PLUGIN" << std::endl
+			<< "if [ $? -ne 0 ] ; then" << std::endl
+			<< "	exit 1" << std::endl
+			<< "fi" << std::endl
+			<< "$PLUGIN" << std::endl
+			<< "if [ $? -ne 0 ] ; then" << std::endl
+			<< "	exit 1" << std::endl
+			<< "fi" << std::endl
+			<< "$(cat $(basename " << _nom << ") | ${SVM_PATH}/svm_config -p directory)/plugin_install -l" << (_joue_tests?" check":"") << ((not _garde_construction)?" dist":"") << std::endl
+			<< ")" << std::endl
+			<< "if [ $? -ne 0 ] ; then" << std::endl
+			<< "	error \"Build of plugin " << _nom << " failed.\"" << std::endl
+			<< "fi" << std::endl;
+	}
+}
+
+void ContenuPaquetFichier::generation(std::ostream& os)
+{
+	FichierPaquet::generation(os);
+}
+
+void ContenuPaquetCode::generation(std::ostream& os)
+{
+	os	<< "(" << std::endl
+		<< _code << ")" << std::endl;
+}
+
+void Paquet::verification()
+{
+	for(auto o: _options)
+	{
+		o->verification();
+	}
+	for(auto c: _contenus)
+	{
+		c->verification();
+	}
+}
+
+void Paquet::construction(const bool debug)
+{
+	ConfigurationPaquet configuration;
+	configuration.variable(AttributPaquet ("only_in_dev",{std::make_shared<ValeurOptionBooleen>(debug)}));
+	configuration.variable(AttributPaquet ("only_in_release",{std::make_shared<ValeurOptionBooleen>(not debug)}));
+	for(auto o: _options)
+	{
+		o->construction(configuration);
+	}
+	_test_version_api_svm = configuration.lit_booleen("svm_api_version_check",0);
+	_version = configuration.lit_chaine("version",0);
+	_auteur = configuration.lit_chaine("author",0);
+	_mainteneur = configuration.lit_chaine("maintainer",0) + " <" + configuration.lit_chaine("maintainer",1) + ">";
+	for(auto c: _contenus)
+	{
+		c->construction(configuration);
+	}
+}
+
+void Paquet::generation(const std::string& fichier_sortie)
+{
+	std::string repertoire = _nom + "-" + _version;
+	SortiePaquet sp(fichier_sortie);
+	sp()	<< "#!/bin/bash" << std::endl;
+	if(not _licence.empty())
+	{
+		sp()	<< "#" << std::endl;
+		std::istringstream iss;
+		iss.str(_licence);
+		for(std::string ligne ; std::getline(iss, ligne) ; )
+		{
+			sp()	<< "# " << ligne << std::endl;
+		}
+		sp()	<< "#" << std::endl;
+	}
+	sp()	<< std::endl
+		<< "SVM_PATH=$(dirname $(which svm))" << std::endl
+		<< "SVM_SOURCE_API_VERSION=\"" << SVM::Global::Installation::valeur("version_api") << "\"" << std::endl
+		<< "LOGS=1" << std::endl
+		<< "RESET=0" << std::endl
+		<< "log()" << std::endl
+		<< "{" << std::endl
+		<< "	if [ ${LOGS} -eq 1 ] ; then" << std::endl
+		<< "		echo \"LOG  : $@\" >&2" << std::endl
+		<< "	fi" << std::endl
+		<< "}" << std::endl
+		<< "error()" << std::endl
+		<< "{" << std::endl
+		<< "	echo \"ERROR: $@\" >&2" << std::endl
+		<< "	exit 1" << std::endl
+		<< "}" << std::endl
+		<< "while getopts s:qfhv OPT ;" << std::endl
+		<< "do" << std::endl
+		<< "	case $OPT in" << std::endl
+		<< "		s)" << std::endl
+		<< "			SVM_PATH=$OPTARG" << std::endl
+		<< "		;;" << std::endl
+		<< "		q)" << std::endl
+		<< "			LOGS=0" << std::endl
+		<< "		;;" << std::endl
+		<< "		f)" << std::endl
+		<< "			RESET=1" << std::endl
+		<< "		;;" << std::endl
+		<< "		v)" << std::endl
+		<< "			cat >&2 << END" << std::endl
+		<< "Package " << _nom << " version " << _version << "." << std::endl
+		<< std::endl
+		<< "Built on SVM API version " << SVM::Global::Installation::valeur("version_api") << "." << std::endl
+		<< "Maintained by " << _mainteneur << "." << std::endl
+		<< std::endl
+		<< _licence << std::endl
+		<< "END" << std::endl
+		<< "			exit 0" << std::endl
+		<< "		;;" << std::endl
+		<< "		*)" << std::endl
+		<< "			cat >&2 << EOM" << std::endl
+		<< "$0 [ options ]" << std::endl
+		<< std::endl
+		<< "Package " << _nom << " installer." << std::endl
+		<< std::endl
+		<< "Options:" << std::endl
+		<< "	-s <dir>: SVM binary installation directory." << std::endl
+		<< "	-f      : Force installation by removing installation target before reinstallation." << std::endl
+		<< "	-q      : Suppress installer logs." << std::endl
+		<< "	-h      : Display this help." << std::endl
+		<< "	-v      : Display package version and information." << std::endl
+		<< "EOM" << std::endl
+		<< "			exit 0" << std::endl
+		<< "		;;" << std::endl
+		<< "	esac" << std::endl
+		<< "done" << std::endl
+		<< "if [ ! -f $SVM_PATH/svm ] ; then" << std::endl
+		<< "	error \"SVM binary directory is invalid.\"" << std::endl
+		<< "fi" << std::endl
+		<< "if [ $RESET -eq 1 ] ; then" << std::endl
+		<< "	rm -rf " << repertoire << std::endl
+		<< "fi" << std::endl
+		<< "if [ -d " << repertoire << " ] ; then" << std::endl
+		<< "	error \"Installation directory " << repertoire << " already exist.\"" << std::endl
+		<< "fi" << std::endl
+		<< "mkdir -p " << repertoire << std::endl
+		<< "if [ ! -d " << repertoire << " ] ; then" << std::endl
+		<< "	error \"Impossible to create installation directory " << repertoire << " \"" << std::endl
+		<< "fi" << std::endl
+		<< "cd " << repertoire << std::endl;
+	if(_test_version_api_svm)
+	{
+		sp()	<< "log \"Checking SVM API version compatibility.\"" << std::endl
+			<< "if [ $($SVM_PATH/svm_config -c version_api) != \"" << SVM::Global::Installation::valeur("version_api") << "\" ] ; then" << std::endl
+			<< "	error \"SVM API version for installation has incompatible API version (Version " << SVM::Global::Installation::valeur("version_api") << " required).\"" << std::endl
+			<< "fi" << std::endl;
+	}
+	for(auto c: _contenus)
+	{
+		c->generation(sp());
+	}
+	sp() << "log \"Installation of package " << _nom << " version " << _version << " complete in '" << repertoire << "'.\"" << std::endl;
+	sp.finalise();
+}
